/* $Id$
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

/// @file Region_boundary_.hpp
/// Data storage class.
///
/// This file was generated by application DATATOOL
/// using the following specifications:
/// 'mmdb3.asn'.
///
/// ATTENTION:
///   Don't edit or commit this file into CVS as this file will
///   be overridden (by DATATOOL) without warning!

#ifndef OBJECTS_MMDB3_REGION_BOUNDARY_BASE_HPP
#define OBJECTS_MMDB3_REGION_BOUNDARY_BASE_HPP

// standard includes
#include <serial/serialbase.hpp>
BEGIN_NCBI_SCOPE

#ifndef BEGIN_objects_SCOPE
#  define BEGIN_objects_SCOPE BEGIN_SCOPE(objects)
#  define END_objects_SCOPE END_SCOPE(objects)
#endif
BEGIN_objects_SCOPE // namespace ncbi::objects::


// forward declarations
class CBrick;
class CCone;
class CCylinder;
class CSphere;


// generated classes

/////////////////////////////////////////////////////////////////////////////
/// Region boundaries are defined by regular solids located in the model space.  
class NCBI_MMDB3_EXPORT CRegion_boundary_Base : public CSerialObject
{
    typedef CSerialObject Tparent;
public:
    // constructor
    CRegion_boundary_Base(void);
    // destructor
    virtual ~CRegion_boundary_Base(void);

    // type info
    DECLARE_INTERNAL_TYPE_INFO();


    /// Choice variants.
    enum E_Choice {
        e_not_set = 0,  ///< No variant selected
        e_Sphere,
        e_Cone,
        e_Cylinder,
        e_Brick
    };
    /// Maximum+1 value of the choice variant enumerator.
    enum E_ChoiceStopper {
        e_MaxChoice = 5 ///< == e_Brick+1
    };

    /// Reset the whole object
    virtual void Reset(void);

    /// Reset the selection (set it to e_not_set).
    virtual void ResetSelection(void);

    /// Which variant is currently selected.
    E_Choice Which(void) const;

    /// Verify selection, throw exception if it differs from the expected.
    void CheckSelected(E_Choice index) const;

    /// Throw 'InvalidSelection' exception.
    NCBI_NORETURN void ThrowInvalidSelection(E_Choice index) const;

    /// Retrieve selection name (for diagnostic purposes).
    static string SelectionName(E_Choice index);

    /// Select the requested variant if needed.
    void Select(E_Choice index, EResetVariant reset = eDoResetVariant);
    /// Select the requested variant if needed,
    /// allocating CObject variants from memory pool.
    void Select(E_Choice index,
                EResetVariant reset,
                CObjectMemoryPool* pool);

    // types
    typedef CSphere TSphere;
    typedef CCone TCone;
    typedef CCylinder TCylinder;
    typedef CBrick TBrick;

    // getters
    // setters

    // typedef CSphere TSphere
    bool IsSphere(void) const;
    const TSphere& GetSphere(void) const;
    TSphere& SetSphere(void);
    void SetSphere(TSphere& value);

    // typedef CCone TCone
    bool IsCone(void) const;
    const TCone& GetCone(void) const;
    TCone& SetCone(void);
    void SetCone(TCone& value);

    // typedef CCylinder TCylinder
    bool IsCylinder(void) const;
    const TCylinder& GetCylinder(void) const;
    TCylinder& SetCylinder(void);
    void SetCylinder(TCylinder& value);

    // typedef CBrick TBrick
    bool IsBrick(void) const;
    const TBrick& GetBrick(void) const;
    TBrick& SetBrick(void);
    void SetBrick(TBrick& value);


private:
    // copy constructor and assignment operator
    CRegion_boundary_Base(const CRegion_boundary_Base& );
    CRegion_boundary_Base& operator=(const CRegion_boundary_Base& );
    // choice state
    E_Choice m_choice;
    // helper methods
    void DoSelect(E_Choice index, CObjectMemoryPool* pool = 0);

    static const char* const sm_SelectionNames[];
    // data
    NCBI_NS_NCBI::CSerialObject *m_object;
};






///////////////////////////////////////////////////////////
///////////////////// inline methods //////////////////////
///////////////////////////////////////////////////////////
inline
CRegion_boundary_Base::E_Choice CRegion_boundary_Base::Which(void) const
{
    return m_choice;
}

inline
void CRegion_boundary_Base::CheckSelected(E_Choice index) const
{
    if ( m_choice != index )
        ThrowInvalidSelection(index);
}

inline
void CRegion_boundary_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset, NCBI_NS_NCBI::CObjectMemoryPool* pool)
{
    if ( reset == NCBI_NS_NCBI::eDoResetVariant || m_choice != index ) {
        if ( m_choice != e_not_set )
            ResetSelection();
        DoSelect(index, pool);
    }
}

inline
void CRegion_boundary_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset)
{
    Select(index, reset, 0);
}

inline
bool CRegion_boundary_Base::IsSphere(void) const
{
    return m_choice == e_Sphere;
}

inline
bool CRegion_boundary_Base::IsCone(void) const
{
    return m_choice == e_Cone;
}

inline
bool CRegion_boundary_Base::IsCylinder(void) const
{
    return m_choice == e_Cylinder;
}

inline
bool CRegion_boundary_Base::IsBrick(void) const
{
    return m_choice == e_Brick;
}

///////////////////////////////////////////////////////////
////////////////// end of inline methods //////////////////
///////////////////////////////////////////////////////////





END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE


#endif // OBJECTS_MMDB3_REGION_BOUNDARY_BASE_HPP
